#ifndef __org_openoffice_da_writer2latex_XW2LStarMathConverter_idl__ 
#define __org_openoffice_da_writer2latex_XW2LStarMathConverter_idl__
#include <com/sun/star/uno/XInterface.idl>

module org { module openoffice { module da { module writer2latex { 

interface XW2LStarMathConverter : com::sun::star::uno::XInterface  
{ 

    // method org::openoffice::da::writer2latex::XW2LStarMathConverter::convertFormula
    string convertFormula ( [in] string sStarMathFormula ); 

    // method org::openoffice::da::writer2latex::XW2LStarMathConverter::getPreamble
    string getPreamble ( ); 

}; 

}; }; }; }; 

#endif

 